/**
 * Author = Robbert Harms
 * Date = 2014-09-21
 * License = LGPL v3
 * Maintainer = Robbert Harms
 * Email = robbert.harms@maastrichtuniversity.nl
 */

/**
 * This file provides a list of MRI specific constants used throughout the compartment CL functions.
 */

/**
 * Gamma represent the gyromagnetic ratio of protons in water (nucleus of H)
 * and are in units of (rad s^-1 T^−1)
 **/
#ifndef GAMMA_H
    #define GAMMA_H 2.675987E8
#endif

/** the square of Gamma as defined above */
#ifndef GAMMA_H_SQ
    #define GAMMA_H_SQ 7.160906424169000e+16
#endif
